/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.inovat.kex.bast.common.datawrapper;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.sys.funclib.debug.Debug;

/*
 * History:
 * 2007-07-13 [KS]  Bugfix wegen Zugriffsfehler in Kernsoftware beim Aufruf von
 *                  Data.getScaledValue(String).shortValue() und Skalierung !=1
 *
 * Revision 1.7  2008/09/04 12:52:13  Logosdev
 * Prfungen mit isState() und isNumber() von getUnscaledValue() auf getScaledValue() umgestellt, da 'Unscaled Values' fr isNumber() immer true liefern (weil auch Zustnde als Zahl ausgelesen werden knnen, was zu Fehlern beim BASt-Export in der DAUZ ST fhrte)
 */


/**
 * kapselt die AttributListe atl.geschwindigkeit.
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author J. Schawaller
 * @version $Id: AtlGeschwindigkeit.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class AtlGeschwindigkeit {

  /** DebugLogger fuer Debug-Ausgaben. */
  private static Debug debug = Debug.getLogger();

  private Data _Data;

  public static final String ATTR_NAME_WERT = "Wert";
  public static final String ATTR_NAME_STATUS = "Status";
  public static final String ATTR_NAME_GUETE = "Gte";

  public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR = "nicht ermittelbar";
  public static final String ZUSTAND_WERT_FEHLERHAFT = "fehlerhaft";
  public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR_FEHLERHAFT = "nicht ermittelbar/fehlerhaft";

  protected boolean _hasState = true;

  public AtlGeschwindigkeit(Data data) {
    _Data = data;
    _hasState = "atl.geschwindigkeit".equals(data.getAttributeType().getPid());
  }

  public void doInitialize() {
    new AtlStatus(_Data.getItem("Status")).doInitialize();
    new AtlGuete(_Data.getItem("Gte")).doInitialize();
    _Data.getScaledValue("Wert").setText(ZUSTAND_WERT_NICHT_ERMITTELBAR);
  }

  public IntegerValueRange getRangeOfGeschwindigkeit() {
    return ValueRangeHashtable.getInstance(this._Data.getItem("Wert")).get("att.geschwindigkeit");
  }
  public double getWert() {
// BUGFIX wegen Exception mit neuer Kernsoftware
//    return _Data.getScaledValue("Wert").shortValue();
    // return (short)(_Data.getScaledValue("Wert").doubleValue());
    return _Data.getScaledValue("Wert").doubleValue();
  }

  public void setWert (double wert) throws IllegalArgumentException {
    if (wert>getRangeOfGeschwindigkeit().getMaximum()) {
      StringBuilder sb = new StringBuilder();
      sb.append("Geschwindigkeitswert ").append(_Data.getName());
      sb.append(" wurde auf ZUSTAND_WERT_FEHLERHAFT gesetzt, da der bergebende Wert ");
      sb.append(wert).append(" das erlaubte Maximum ").append(getRangeOfGeschwindigkeit().getMaximum());
      sb.append(" berschreitet.");
      debug.error(sb.toString());
      throw new IllegalArgumentException(sb.toString());
    }
    try {
      if (wert < 0) {
        _Data.getUnscaledValue("Wert").set((int)wert);
      }
      else {
        _Data.getScaledValue("Wert").set(wert);
        // logger.info(_Data);
      }
    }
    catch (IllegalArgumentException ex) {
      long lMaxValue = getRangeOfGeschwindigkeit().getMaximum();
      long lMinValue = getRangeOfGeschwindigkeit().getMinimum();
      throw new IllegalArgumentException("Geschwindigkeitswert konnte nicht auf " +
                                         wert + " gesetzt werden. (Min/Max:" +
                                         lMinValue+"/"+lMaxValue + ")", ex);
    }
  }


//  public void setWert (int wert) {
//    try {
//      if (wert < 0) {
//        _Data.getUnscaledValue("Wert").set(wert);
//      }
//      else {
//        // _Data.getScaledValue("Wert").set(wert);
//        _Data.getScaledValue("Wert").set( 0.1*wert);
//      }
//    }
//    catch (IllegalArgumentException ex) {
//      ex.printStackTrace();
//      long lMaxValue = getRangeOfGeschwindigkeit().getMaximum();
//      long lMinValue = getRangeOfGeschwindigkeit().getMinimum();
//      throw new IllegalArgumentException("Geschwindigkeitswert konnte nicht auf " +
//                                         wert + " gesetzt werden. (Min/Max:" +
//                                         lMinValue+"/"+lMaxValue + ")");
//    }
//  }
//  public void setWert (short wert) {
//    try {
//      if (wert < 0) {
//        _Data.getUnscaledValue("Wert").set(wert);
//      }
//      else {
//        // _Data.getScaledValue("Wert").set(wert);
//        _Data.getScaledValue("Wert").set(wert);
//      }
//    }
//    catch (IllegalArgumentException ex) {
//      long lMaxValue = getRangeOfGeschwindigkeit().getMaximum();
//      long lMinValue = getRangeOfGeschwindigkeit().getMinimum();
//      throw new IllegalArgumentException("Geschwindigkeitswert konnte nicht auf " +
//                                         wert + " gesetzt werden. (Min/Max:" +
//                                         lMinValue+"/"+lMaxValue + ")");
//    }
//  }
  public IntegerValueState getZustandWert() {
    return _Data.getScaledValue("Wert").getState();
  }

  public void setZustandWert (IntegerValueState integerValueState) {
      _Data.getScaledValue("Wert").setState(integerValueState);
  }
  public void setZustandWert (String zustandsName) {
      _Data.getScaledValue("Wert").setText(zustandsName);
  }

  /**
   * liefert AtlStatus-Instance zur&uuml;ck, wenn das gekapselte Data vom Typ
   * atl.geschwindigkeit ist und einen Status besitzt (sonst null).
   * @return AtlStatus
   */
  public AtlStatus  getStatus() {
    if (_hasState) {
      return new AtlStatus(_Data.getItem("Status"));
    }
    return null;
  }

  /**
   * liefert true zur&uuml;ck, wenn das gekapselte Data vom Typ
   * atl.geschwindigkeit ist und
   * einen Status besitzt.
   *
   * @return boolean
   */
  public boolean hasState() {
    return _hasState;
  }

  public AtlGuete getGuete() {
    return new AtlGuete(_Data.getItem("Gte"));
  }

  // Also: _Data.getUnscaledValue("Wert").isNumber() liefert IMMER true, da sich die Zustnde ja als Zahl abfragen lassen
  //       _Data.getUnscaledValue("Wert").isState()  liefert true falls Zustand
  // eigentlich geht es aber um _Data.getScaledValue("Wert") also:
  //       _Data.getScaledValue("Wert").isNumber() liefert true, wenn es wirklich ein Wert ist
  //       _Data.getScaledValue("Wert").isState()  liefert true, wenn es ein Zustand ist

  public boolean isState() {
//    return _Data.getUnscaledValue("Wert").isState();
    return _Data.getScaledValue("Wert").isState();
  }

  public boolean isNumber() {
//    return !isState();
    return _Data.getScaledValue("Wert").isNumber();
  }


}

