/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;


/*
 * Revision 1.1  2007/07/04 11:43:36  Trier
 * Neuordnung
 */

import java.util.HashMap;
import java.util.TreeMap;


/**
 * kapselt die Bundesl&auml;nder, die einer LangzeitZ&auml;hlStelle zu geordnet ist und deren Code
 * durch die BASt vorgegeben ist.
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: Bundesland.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class Bundesland implements Comparable<Bundesland> {

  private static HashMap<Integer, Bundesland> __HashMapCodeAsKey;

  public static final Bundesland SchleswigHolstein = new Bundesland("Schleswig-Holstein", 1, "SH");
  public static final Bundesland Hamburg = new Bundesland("Hamburg", 2, "HH");
  public static final Bundesland Niedersachsen = new Bundesland("Niedersachsen", 3, "NI");
  public static final Bundesland Bremen = new Bundesland("Bremen", 4, "HB");
  public static final Bundesland NordrheinWestfalen = new Bundesland("Nordrhein-Westfalen", 5, "NW");
  public static final Bundesland Hessen = new Bundesland("Hessen", 6, "HE");
  public static final Bundesland RheinlandPfalz = new Bundesland("Rheinland-Pfalz", 7, "RP");
  public static final Bundesland BadenWuertenberg = new Bundesland("Baden-Wrtenberg", 8, "BW");
  public static final Bundesland Bayern = new Bundesland("Bayern", 9, "BY");
  public static final Bundesland Saarland = new Bundesland("Saarland", 10, "SL");
  public static final Bundesland Berlin = new Bundesland("Berlin", 11, "BE");
  public static final Bundesland Brandenburg = new Bundesland("Brandenburg", 12, "BB");
  public static final Bundesland MecklenburgVorpommern = new Bundesland("Mecklenburg-Vorpommern", 13, "MV");
  public static final Bundesland Sachsen = new Bundesland("Sachsen", 14, "SA");
  public static final Bundesland SachsenAnhalt = new Bundesland("Sachsen-Anhalt", 15, "ST");
  public static final Bundesland Thueringen = new Bundesland("Thringen", 16, "TH");

  private String _Name;
  private int _Code;
  private String _Abbreviation;

  private static Bundesland[] __BundesLaenderArray;

  private Bundesland(String name, int code, String abbreviation) {
    _Name = name;
    _Code = code;
    _Abbreviation = abbreviation;
    if (__HashMapCodeAsKey == null) {
      __HashMapCodeAsKey = new HashMap<Integer, Bundesland>();
    }
    __HashMapCodeAsKey.put(code, this);
  }

  public static Bundesland getInstanceByCode(int code) {
    return __HashMapCodeAsKey.get(code);
  }

  public Bundesland[] getLaender() {
    if (__BundesLaenderArray == null) {
      TreeMap<Integer, Bundesland> map = new TreeMap<Integer, Bundesland>(__HashMapCodeAsKey);
      __BundesLaenderArray = map.values().toArray(new Bundesland[map.size()]);
    }
    return __BundesLaenderArray;
  }

  public String toString() {
    return _Name;
  }

  public String getAbbreviation() {
    return _Abbreviation;
  }

  public int getKennung() {
    return _Code;
  }

  public String getKennungAsString() {
    if (_Code<10) {
      return '0' + String.valueOf(_Code);
    }
    return String.valueOf(_Code);
  }

  public int compareTo(Bundesland b) {
    return b._Name.compareTo(_Name);
  }

}
