/**
 * Copyright (c) 2004-2008 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;

/**
 * History:
 * 2008-10-17  identische Funktionen aus FG03Type und FG06Type => Superklasse
 */

import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.inovat.kex.bast.common.NetzClientDavConnection;

/**
 * <p>Mit dem Objekt wird die Beziehung zwischen einem TLS "Umsetzungsmodul"
 * und der von ihm gelieferten "Attributgruppe" fr den Sensorwert hergestellt.
 * FGdefaultType ist Superklasse fr die FG-spezifischen Implementierungen</p>
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Trier / Strey
 * @version $Id: FGdefaultType.java 442 2008-11-10 06:59:32Z Schawaller $
 */
 public abstract class FGdefaultType extends Object{

	public static final short STATUS_OK = 0;
	public static final short STATUS_WARNING = 1;
	public static final short STATUS_ALERT = 2;

	protected String _Name;
	protected String _Abbreviation;
	protected String _UmsetzungsModul;
	protected int _Type;
	protected AttributeGroup _Atg;
  boolean _IsFahrstreifenSensor;

  /**
   * <p>Initialisiert die lokalen Variablen:</p>
   * 
   * @param umsetzungsModul Klassenname/Konfigurationsangabe fr Umsetzungsmodul
   * @param name Bezeichnung des Sensorwerts z.B. "Lufttemperatur"
   * @param abbreviation Abkrzung des Sensorwerts z.B. "LT"
   * @param type Typ
   * @param atgPid Attributgruppe, mit der das Umsetzungsmodul die Fachdaten
   *   liefert, z.B. "atg.tlsUfdErgebnisMeldungLuftTemperaturLT" 
   */
  public FGdefaultType(String umsetzungsModul, String name, String abbreviation, int type, String atgPid, boolean isFahrstreifenSensor ) {
    _UmsetzungsModul = umsetzungsModul;
    _Name = name;
    _Abbreviation = abbreviation;
    _Type = type;
    _IsFahrstreifenSensor = isFahrstreifenSensor;
    DataModel model = NetzClientDavConnection.getInstance().getDataModel();
    try {
      _Atg = model.getAttributeGroup(atgPid);
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

	/**
	 * <p>Liefert den Namen des Umsetzungsmoduls (Java Klassenname).</p>
	 * @return String
	 */
	public final String getUmsetzungsModul() {
	  return _UmsetzungsModul;
	}

	/**
	 * <p>Liefert die Beschreibung / den Namen des Sensorikwerts.</p>
	 * @return String
	 */
	public final String getName() {
	  return _Name;
	}

	/**
	 * <p>Liefert die Kurzbezeichnung des Sensorikwerts (z.B. "FBT").</p>
	 * @return String
	 */
	public final String getAbbreviation() {
	  return _Abbreviation;
	}

  /**
   * <p>Liefert den bei der Initialisierung angegebenen Typ.</p>
   * @return int
   */
	public final int getType() {
	  return _Type;
	}

  /**
   * <p>Liefert die Attributgruppe, mit der der Sensorikwert im Datenverteiler
   * / der Konfiguration abgebildet wird.</p>
   * @return AttributeGroup
   */
	public final AttributeGroup getAtg() {
	  return _Atg;
	}

  /**
   * <p>Liefert <code>true</code> falls der Sensorwert fahrbahnbezogen ist,
   * ansonsten <code>false</code></p>
   * 
   * @return
   */
  public boolean isFahrstreifenSensor() {
    return _IsFahrstreifenSensor;
  }

	public String toString() {
	  StringBuilder sb = new StringBuilder(getClass().getName());
	  sb.append(_UmsetzungsModul).append(" - ");
	  sb.append(_Name).append(" - ");
	  sb.append(_Abbreviation).append(" - ");
	  sb.append(_Type).append(" - ");
	  sb.append(_Atg.getPid());
	  sb.append(")");
	  return sb.toString();
	}

}