/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;

import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.inovat.kex.bast.common.*;

/**
 * kapselt den DatenModellTyp verkehrsDatenQuelleASC
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: FahrstreifenQuelleASCII.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class FahrstreifenQuelleASCII extends ConfigurationObjectWrapper implements FahrstreifenQuelle {

  private static ConfigurationObjectType __ConfigurationObjectType;
  private static AttributeGroup __AtgVerkehrsDatenQuelleASC;

  private String _PID;
  private String _URL;
  private String _DatenFormat;
  private String _VKRaster;
  private int _FahrStreifenIndex;

  public String getURL() {
    return _URL;
  }
  public int getFahrStreifenIndex() {
    return _FahrStreifenIndex;
  }
  public String getDatenFormat() {
    return _DatenFormat;
  }
  public String getVKRaster() {
    return _VKRaster;
  }


  protected FahrstreifenQuelleASCII(String pid) throws DataModellMismatchException, DataModellObjectNotFoundException {
    super(pid);
  }


  protected FahrstreifenQuelleASCII(ConfigurationObject confObject) throws DataModellMismatchException {
    super(confObject);
  }

  protected void initData(ConfigurationObject confObject) {
    Data oData = confObject.getConfigurationData(__AtgVerkehrsDatenQuelleASC);
    _URL = oData.getTextValue("Url").getText();
    _DatenFormat = oData.getTextValue("DatenFormat").getText();
    _VKRaster = oData.getTextValue("VKRaster").getText();
    _FahrStreifenIndex = oData.getUnscaledValue("FahrStreifenIndex").intValue();
  }

  /**
   * gibt ein Array aller Objekte zurck, die vom Typ typ.verkehrsDatenQuelleASC
   * sind.
   *
   * @return FahrstreifenQuelleASCII[]
   * @throws DataModellMismatchException
   */
  public static FahrstreifenQuelleASCII[] getAll() throws DataModellMismatchException {
    List<SystemObject> list = NetzClientDavConnection.getInstance().getDataModel().getType("typ.verkehrsDatenQuelleASCII").getElements();
    FahrstreifenQuelleASCII[] retValue = new FahrstreifenQuelleASCII[list.size()];
    for (int i = 0; i < list.size(); i++) {
      retValue[i] = new FahrstreifenQuelleASCII( (ConfigurationObject) list.get(i));
    }
    return retValue;
  }


  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("PID: \"").append(_PID);
    sb.append("\"  URL: \"").append(_URL);
    sb.append("\"  DatenFormat: \"").append(_DatenFormat);
    sb.append("\"  VKRaster: \"").append(_VKRaster);
    sb.append("\"  FahrStreifenIndex: \"").append(_FahrStreifenIndex);
    sb.append("\"");
    return sb.toString();
  }

  public String getTreeString() {
    return getName();
  }

  ConfigurationObjectType getType() {
    if (__ConfigurationObjectType==null) {
      DataModel model = NetzClientDavConnection.getInstance().getDataModel();
      __ConfigurationObjectType = (ConfigurationObjectType)model.getType("typ.verkehrsDatenQuelleASCII");
      __AtgVerkehrsDatenQuelleASC = model.getAttributeGroup("atg.verkehrsDatenQuelleASCII");
      List<SystemObject> list = __ConfigurationObjectType.getElements();
      model.getConfigurationData(list, __AtgVerkehrsDatenQuelleASC);
    }
    return __ConfigurationObjectType;
  }


}
