/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;


/*
 * Revision 1.4  2007/11/26 12:10:20  Trier
 * <Kein Kommentar eingegeben>
 */

import java.util.ArrayList;
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.inovat.kex.bast.common.*;

/**
 * kapselt den DatenModellTyp steuerModul
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: SteuerModul.java 1292 2009-03-20 13:12:46Z Trier $
 */
public class SteuerModul extends ConfigurationObjectWrapper {

//  private static final de.inovat.kex.bast.funclib.logger.Logger logger =
//    de.inovat.kex.bast.funclib.logger.LoggerFactory.getLoggerFactory().getLogger(SteuerModul.class);


  private static WeakHashMap<String,SteuerModul> __Cache = new WeakHashMap<String,SteuerModul>();

  private static ConfigurationObjectType __ConfigurationObjectType;
  private static ConfigurationObjectType __ConfigurationObjectTypeOfEAK;
  private static AttributeGroup __AtgGeraet;

  private String _Bezeichnung;
  private int _KnotenNummer;
  private String _UmsetzungsModul;

  private ArrayList<EAK> _EAKList;

  private DEKanal _DEKanal;
  private KriBus _Bus;


  /**
   * setzt den Bus &uuml;ber den das SteuerModul am Kri angeschlossen ist.
   * 
   * @param bus
   */
  public void setBus(KriBus bus) {
    _Bus = bus;
  }

  /**
   * gibt den Bus zur&uuml;ck, &uuml;ber den das SteuerModul am Kri angeschlossen ist.
   * 
   * @return KriBus
   */
  public KriBus getBus() {
    return _Bus;
  }

  /**
   * gibt die Bezeichnung des EAK zur&uuml;ck.
   *
   * @return String
   */
  public String getBezeichnung() {
    return _Bezeichnung;
  }

  /**
   * gibt den DE-Kanal zum Steuern des SteuerModuls zur&uuml;ck.
   *
   * @return DEKanal
   */
  public DEKanal getSteuerDEKanal() {
    return _DEKanal;
  }


  /**
   * gibt die Knotennummer (OSI-7-Adresse) des Ger&auml;tes zur&uuml;ck.
   *
   * @return die Knotennummer;
   */
  public int getKnotenNummer() {
    return _KnotenNummer;
  }

  /**
   * gibt  das fr die Umsetzung des Protokolls (Lesen und Schreiben) verwendete
   * Modul an diesem Knoten zur&uuml;ck.
   *
   * @return Klassennamen des UmsetzungsModuls.
   */
  public String getUmsetzungsModul() {
    return _UmsetzungsModul;
  }

  /**
   * gibt die EAK's zur&uuml;ck oder null, wenn keine angeschlossen sind.
   *
   * @return EAK's.
   */
  public EAK[] getEAKs() {
    if ((_EAKList!=null) && (_EAKList.size()>0)) {
      return _EAKList.toArray(new EAK[_EAKList.size()]);
    }
    else {
      return null;
    }
  }


  /**
   * <p>gibt den Sammelkanal des entsprechenen Typs zur&uuml;ck</p>
   *
   * @param classOfdeKanal {@link DEKanal.Lve}, {@link DEKanal.Axl}, {@link
   *   DEKanal.Ufd}, {@link DEKanal.Wzg} oder {@link DEKanal.Vlt},
   * @return Sammelkanl f&uuml;r entsprechene Funktionsgruppe
   */
  public <K extends DEKanal> K getSammelKanal(Class<K> classOfdeKanal) {

    DEKanal deKanal = null;
    for (int eakNr=0; ((eakNr<_EAKList.size()) && (deKanal==null)); eakNr++) {
      EAK eak = _EAKList.get(eakNr);
      DEKanal[] deArray = eak.getDEKanals();
      for (int deNr=0; ((deNr<deArray.length) && (deKanal==null)); deNr++) {
        DEKanal tmpDEKanal = deArray[deNr];
        if ((tmpDEKanal.getDEKanal()==255) && (classOfdeKanal.isInstance(tmpDEKanal))) {
          deKanal = tmpDEKanal;
        }
      }
    }
    return classOfdeKanal.cast(deKanal);
  }

  /**
   * <p>gibt alle Sammelkan&auml;le zur&uuml;ck, die an dem Steuermodul
   * angeschlossen sind.</p>
   *
   * @return DEKanal[]
   */
  public DEKanal[] getSammelKanaele() {
    ArrayList<DEKanal> list = new ArrayList<DEKanal>(1);
    for (int eakNr=0; eakNr<_EAKList.size(); eakNr++) {
      EAK eak = _EAKList.get(eakNr);
      DEKanal[] deArray = eak.getDEKanals();
      for (int deNr=0; deNr<deArray.length; deNr++) {
        DEKanal tmpDEKanal = deArray[deNr];
        if (tmpDEKanal.getDEKanal()==255) {
          list.add(tmpDEKanal);
        }
      }
    }
    return list.toArray( new DEKanal[list.size()]);
  }


  /**
   * erzeugt das WrapperObject f&uuml;r das Object mit der angegebenen Pid
   *
   * @see ConfigurationObjectWrapper#ConfigurationObjectWrapper(String pid)
   *
   * @param pid String
   * @throws DataModellMismatchException
   * @throws DataModellObjectNotFoundException
   */
  private SteuerModul(String pid) throws DataModellMismatchException, DataModellObjectNotFoundException {
    super(pid);
  }

  /**
   * erzeugt das WrapperObject f&uuml;r das angegebene Object.
   *
   * @see ConfigurationObjectWrapper#ConfigurationObjectWrapper(ConfigurationObject confObj)
   *
   * @param confObj ConfigurationObject, das ein Steuermodul repr&auml;sentiert.
   * @throws DataModellMismatchException
   */
  private SteuerModul(ConfigurationObject confObj) throws DataModellMismatchException {
    super(confObj);
  }

  protected void initData(ConfigurationObject confObj) throws DataModellMismatchException {

    // virtueller EAK (254 OsiAddr==0)
    EAK virtEAK = null;

    Data oData = confObj.getConfigurationData(__AtgGeraet);
    if (oData != null) {
      _Bezeichnung = oData.getTextValue("Bezeichnung").getText();
      _KnotenNummer = oData.getUnscaledValue("KnotenNummer").intValue();
      _UmsetzungsModul = oData.getTextValue("UmsetzungsModul").getText();
    }
    ObjectSet os = confObj.getObjectSet("Eak");
    if (os != null) {
      List<SystemObject> list = os.getElements();
      if ((list!=null) && (list.size()>0)) {
        _EAKList = new ArrayList<EAK>(list.size());
        for (int i = 0; i < list.size(); i++) {
          ConfigurationObject eakObj = (ConfigurationObject)list.get(i);
          if (eakObj.getType().equals(__ConfigurationObjectTypeOfEAK)) {
            EAK eak = EAK.getInstance(eakObj);
            eak.setSteuerModul(this);
            _EAKList.add(eak);
            if (eak.getOSI2Adresse()==0) {
              virtEAK = eak;
            }
          }
        }
      }
    }
    // Zuordnung der Kontroll-DEKanale zum SM bzw. den EAK
    if (virtEAK!=null) {
      for (int i=0; i<_EAKList.size(); i++) {
        EAK eak = _EAKList.get(i);
        if (eak!=virtEAK) {
          DEKanal deKanal = virtEAK.findDEKanal(eak.getOSI2Adresse());
          eak.setDEKanal(deKanal);
        }
      }
    }
    _DEKanal = (virtEAK!=null) ?  virtEAK.findDEKanal(0) : null;
    __Cache.put(this.getPID(), this);
  }

  ConfigurationObjectType getType() {
    return __ConfigurationObjectType;
  }

  public String getTreeString() {
    return getName();
  }

  /**
   * gibt ein Array aller Objekte zurck, die vom Typ typ.steuerModul sind.
   *
   * @return Unterzentrale[]
   * @throws DataModellMismatchException
   */
  public static SteuerModul[] getAll() throws DataModellMismatchException {
    if (__ConfigurationObjectType == null) {
      init();
    }
    SteuerModul[] retValue;
    if (__ConfigurationObjectType!=null) {
      List<SystemObject> list = __ConfigurationObjectType.getElements();
      retValue = new SteuerModul[list.size()];
      for (int i = 0; i < list.size(); i++) {
        ConfigurationObject confObj = (ConfigurationObject) list.get(i);
        SteuerModul sm = __Cache.get(confObj.getPid());
        if (sm == null) {
          retValue[i] = getInstanceIntern((ConfigurationObject) list.get(i));
        } else {
          retValue[i] = sm;
        }
      }
    }
    else {
      retValue = new SteuerModul[0];
    }
    return retValue;
  }

  /**
   * gibt das SteuerModul mit der &uuml;bergebenen Pid zur&uuml;ck oder null,
   * wenn es kein SteuerModul mit der Pid gibt.
   *
   * @param pid String
   * @throws DataModellMismatchException
   * @return SteuerModul
   */
  public static SteuerModul getInstance(String pid) throws DataModellMismatchException {
    SteuerModul sm = __Cache.get(pid);
    if (sm==null) {
      Object object = NetzClientDavConnection.getInstance().getDataModel().getObject(pid);
      if ((object!=null) && (object instanceof ConfigurationObject)) {
        sm = getInstanceIntern((ConfigurationObject)object);
      }
    }
    return sm;
  }

  /**
   * <p>Liefert das Wrapper-Objekt zum angegebenen Konfigurationsobjekt.</p>
   *
   * @param confObj ConfigurationObject
   * @return SteuerModul
   * @throws DataModellMismatchException
   */
  static public SteuerModul getInstance(ConfigurationObject confObj) throws DataModellMismatchException {
    SteuerModul sm = __Cache.get(confObj.getPid());
    if (sm!=null) {
      return sm;
    }
    else {
      return getInstanceIntern(confObj);
    }
  }

  static synchronized private SteuerModul getInstanceIntern(ConfigurationObject confObj) throws DataModellMismatchException {
    SteuerModul sm = __Cache.get(confObj.getPid());
    if (sm!=null) {
      return sm;
    }
    else {
      if (__ConfigurationObjectType == null) {
        init();
      }
      return new SteuerModul(confObj);
    }
  }

  /**
   * <p>Initialisiert allgemeine Konfigurationsdaten f&uuml;r den typ.steuerModul des Datenmodells.</p>
   */
  private static synchronized void init() {
    if (__ConfigurationObjectType==null) {
      DataModel model = NetzClientDavConnection.getInstance().getDataModel();
      __ConfigurationObjectType = (ConfigurationObjectType)model.getType("typ.steuerModul");
      __AtgGeraet = model.getAttributeGroup("atg.gert");
      __ConfigurationObjectTypeOfEAK = (ConfigurationObjectType)model.getType("typ.eak");

      List<SystemObject> list = __ConfigurationObjectType.getObjects();
      model.getConfigurationData(list, __AtgGeraet);
    }
  }


}
