/**
 * Copyright (c) 2004-2009 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.kex.bast.exporter;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.sys.funclib.debug.Debug;
import de.inovat.kex.bast.KonstantenBast.BAST_FORMAT;
import de.inovat.kex.bast.SendeBAStStatusDaten;
import de.inovat.kex.bast.common.*;
import de.inovat.kex.bast.common.objectwrapper.*;

import java.util.Calendar;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * erm&ouml;glicht die Ausgabe von Z&auml;hlstellendaten in das BASt-Format.
 *
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author R. Trier
 * @version $Id: BastExporter.java 831 2009-01-30 12:54:22Z Strey $
 */

/**
 * <p></p>
 *
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Strey
 * @version $Id: BastExporter.java 831 2009-01-30 12:54:22Z Strey $
 *
 */
public class BastExporter implements ResultDataChannel {

    /** DebugLogger fuer Debug-Ausgaben. */
    private static Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    private AbstractDataChannelBASt[] _Channels;
    private DataDescription           _DataDescription;
    private DataDescription           _DataDescriptionTLSAxlEinzelErgebnisMeldungAchsLastenUKfzGesGewichtV3;
    private DataDescription           _DataDescriptionTlsLveKfz;
    private DataDescription           _DataDescriptionV;  // Datenversion 24 fr Geschwindigkeiten

    /** Datenverteilerverbindung */
    private ClientDavInterface _dav;
    private BAST_FORMAT        _Format;
    private ResultDataProvider _ResultDataProvider;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    public BastExporter(BAST_FORMAT format, ClientDavInterface dav, SendeBAStStatusDaten bastStatusDaten) throws Exception {
        _dav = dav;

        DataModel model = _dav.getDataModel();

        _Format = format;

        switch (format) {
        case V2004 : {
            _Channels = new AbstractDataChannelBASt[] { new DataChannelBAStV2_0(bastStatusDaten) };

            break;
        }

        // Format Einzeldaten Achslast BASt.
        case AXL_EF1 : {
            _Channels = new AbstractDataChannelBASt[] { new DataChannelAxlBAStEF1(DataChannelAxlBAStEF1.IN_RICHTUNG, _dav, bastStatusDaten) };

            break;
        }

        case V2007 : {
            _Channels = new AbstractDataChannelBASt[] { new DataChannelBAStVV2007(bastStatusDaten) };

            break;
        }
        }

        _ResultDataProvider = new ResultDataProvider();
        _ResultDataProvider.setIncrement(1, Calendar.HOUR);
        _ResultDataProvider.addChannel(this);

        //
        AttributeGroup atg    = model.getAttributeGroup("atg.verkehrsDatenLangZeitIntervall");
        Aspect         aspect = model.getAspect("asp.messWertErsetzung");

        _DataDescription = new DataDescription(atg, aspect, (short) 0);

        //
        atg                       = model.getAttributeGroup("atg.tlsLveKfzEinzelDaten");
        aspect                    = model.getAspect("asp.tlsAntwort");
        _DataDescriptionTlsLveKfz = new DataDescription(atg, aspect, (short) 0);

        //
        atg               = model.getAttributeGroup("atg.tlsLveErgebnisMeldungVersion24");
        aspect            = model.getAspect("asp.tlsAntwort");
        _DataDescriptionV = new DataDescription(atg, aspect, (short) 0);

        //
        AttributeGroup atgTLSAxlEinzelErgebnisMeldungAchsLastenUndKfzGesamtGewichteVersion3 = model.getAttributeGroup("atg.tlsAxlEinzelErgebnisMeldungAchsLastenUndKfzGesamtGewichteVersion3");
        Aspect         aspectTlsAntwort                                                     = model.getAspect("asp.tlsAntwort");

        _DataDescriptionTLSAxlEinzelErgebnisMeldungAchsLastenUKfzGesGewichtV3 = new DataDescription(atgTLSAxlEinzelErgebnisMeldungAchsLastenUndKfzGesamtGewichteVersion3, aspectTlsAntwort, (short) 0);
    }

    //~ METHODEN ==============================================================

    /**
     * @see de.inovat.kex.bast.common.ResultDataChannel#close()
     */
    public void close() {
        try {
            for (AbstractDataChannelBASt _Channel : _Channels) {
                _Channel.close();
            }
        }
        catch (ChannelException ex) {
            ex.printStackTrace();
        }
    }

    /**
     * <p>Initialisiert die Ausgabekanle und den ResultDataProvider, startet die
     * Datenausgabe und schliet danach die Ausgabekanle.</p>
     * <p>Je nach gewhltem BASt-Format unterscheiden die bentigten Datenquellen
     * und damit die dem ResultDataProvider anzugebenden Objekte und
     * Datenbeschreibungen:
     * <ul>
     *   <li>BAST_FORMAT_AXL (Achslastdsaten): Datenquelle ist der DE-Kanal und
     *       die Daten werden aus der TLS-ATG fr Achslastdaten geholt.</li>
     *   <li>BAST_FORMAT_VV2007 (spezielle Geschwindigkeitsdaten): Datenquelle ist
     *       der DE-Kanal und die Daten werden aus der TLS-ATG fr Datenversion 24
     *       geholt (nur an spezielle Geschwindigkeitsmessstellen vorhanden).</li>
     *   <li>sonstige Formate: Datenquelle ist der Fahrstreifen und die Daten
     *       kommen aus den fahrstreifenbezogenen Stundendaten, dabei kann sich
     *       der Aspekt je nach Format noch unterscheiden.</li>
     * </ul><p>
     *
     * @param lzz LangzeitZhlstelle.
     * @throws DataProviderException .
     * @throws ChannelException   .
     */
    private void export(LangZeitZaehlStelle lzz) throws DataProviderException, ChannelException {
        debug.finer(String.format("export( Zhlstelle\"%s\")", lzz));
        _ResultDataProvider.clearDataCombinations();

        Fahrstreifen[] fsArray = lzz.getFahrstreifen();

        if ((fsArray != null) && (fsArray.length > 0)) {
            for (Fahrstreifen fahrstreifen : fsArray) {
                if (_Format == BAST_FORMAT.AXL_EF1) {
                    _ResultDataProvider.setAchslastExportEF1(true);

                    try {
                        if (fahrstreifen.isAchslastErfassung()) {
                            _ResultDataProvider.addDataCombination(fahrstreifen.getDEKanalAxl().getConfigurationObject(), _DataDescriptionTLSAxlEinzelErgebnisMeldungAchsLastenUKfzGesGewichtV3);
                        } else {

                            // Falls auf dem Fahrstreifen keine Achslastdaten vorhanden (FS 3),
                            // werden Verkehrsdaten (FG1 Einzelfahrzeugdaten) abgeholt:
                            if (fahrstreifen.getFahrstreifenQuelle() != null) {
                                ConfigurationObject deKanal = ((DEKanal) (fahrstreifen.getFahrstreifenQuelle())).getConfigurationObject();

                                if ((deKanal != null)) {
                                    _ResultDataProvider.addDataCombination(deKanal, _DataDescriptionTlsLveKfz);
                                } else {
                                    throw new ChannelException("Fr den Fahrstreifen " + fahrstreifen.getName() + "(" + fahrstreifen.getPID() + ") " + "sind keine BASt-Geschwindigkeitsdaten verfgbar.");
                                }
                            }
                        }
                    }
                    catch (DataModellMismatchException ex) {
                        throw new ChannelException("Fr den Fahrstreifen " + fahrstreifen.getName() + "(" + fahrstreifen.getPID() + ") " + "sind keine Achslastdaten verfgbar.");
                    }
                } else if (_Format == BAST_FORMAT.V2007) {
                    if (fahrstreifen.getFahrstreifenQuelle() != null) {
                        ConfigurationObject deKanal = ((DEKanal) (fahrstreifen.getFahrstreifenQuelle())).getConfigurationObject();

                        if ((deKanal != null)) {

                            // TODO sobald realistische Daten vorliegen!!!   if ((deKanal != null) && deKanal.isOfType("typ.deLveGeschwindigkeiten")) {
                            _ResultDataProvider.addDataCombination(deKanal, _DataDescriptionV);
                        } else {
                            throw new ChannelException("Fr den Fahrstreifen " + fahrstreifen.getName() + "(" + fahrstreifen.getPID() + ") " + "sind keine BASt-Geschwindigkeitsdaten verfgbar.");
                        }
                    }
                } else {
                    _ResultDataProvider.addDataCombination(fahrstreifen.getConfigurationObject(), _DataDescription);
                }
            }

            _ResultDataProvider.processData();
        }

        for (AbstractDataChannelBASt _Channel : _Channels) {
            _Channel.close();
        }
    }

    /**
     * <p>exportiert die Daten der Z&auml;hlstelle.</p>
     *
     * <p>Ist das File ein vorhandenes Verzeichnis wird der Dateiname der
     * Ausgabedatei durch den benutzten Channel bestimmt. Vorhandene Dateien
     * werden &uuml;berschrieben.</p>
     *
     * <p>Ist das File kein vorhandenes Verzeichnis wird zur Augabe das File
     * benutzt. Existstiert die Datei, wird diese &uuml;berschrieben.
     *
     * @param lzz LangZeitZaehlStelle, dessen Daten ausgegeben werden sollen.
     * @param anfCalendar Zeitpunkt, ab dem die Daten ausgegeben werden sollen.
     * @param endCalendar Zeitpunkt, bis zu dem die Daten ausgegeben werden
     *   sollen (exclusiv).
     * @throws ChannelException   .
     * @throws DataProviderException  .
     */
    public void export(LangZeitZaehlStelle lzz, Calendar anfCalendar, Calendar endCalendar) throws ChannelException, DataProviderException {
        if (FTPServerConnection.getInstance().isAchslastExport()) {
            _ResultDataProvider.setIncrement(5, 1);
        }

        for (AbstractDataChannelBASt _Channel : _Channels) {
            _Channel.reset(lzz, anfCalendar.get(Calendar.YEAR), anfCalendar.get(Calendar.MONTH) + 1, anfCalendar.get(Calendar.DAY_OF_MONTH));
        }

        // ResultDataProvider ZeitBereich setzen
        // Calendar calender01 = (Calendar)anfCalendar.clone();
        long lDiffInMilllis = endCalendar.getTimeInMillis() - anfCalendar.getTimeInMillis();
        int  iDiffInMin     = (int) (lDiffInMilllis / 1000 / 60);

        _ResultDataProvider.setPeriod(anfCalendar, iDiffInMin, Calendar.MINUTE);
        export(lzz);
    }

    /**
     * <p>exportiert die Daten der Z&auml;hlstelle.</p>
     *
     * <p>Ist das File ein vorhandenes Verzeichnis wird der Dateiname der
     * Ausgabedatei durch den benutzten Channel bestimmt. Vorhandene Dateien
     * werden &uuml;berschrieben.</p>
     *
     * <p>Ist das File kein vorhandenes Verzeichnis wird zur Augabe das File
     * benutzt. Existstiert die Datei, wird diese &uuml;berschrieben.
     *
     * @param lzz LangZeitZaehlStelle, dessen Daten ausgegeben werden sollen.
     * @param year Jahr f&uuml;r das die Ausgabe erfolgen soll.
     * @param month Monat f&uuml;r den bzw. das die Ausgabe erfolgen soll (Januar=0).
     *
     * @throws ChannelException .
     * @throws DataProviderException  .
     */
    public void export(LangZeitZaehlStelle lzz, int year, int month) throws ChannelException, DataProviderException {
        for (AbstractDataChannelBASt _Channel : _Channels) {
            _Channel.reset(lzz, year, month);
        }

        // ResultDataProvider ZeitBereich setzen
        Calendar calender = Calendar.getInstance();

        calender.clear();
        calender.set(Calendar.YEAR, year);
        calender.set(Calendar.HOUR, 0);
        calender.set(Calendar.MINUTE, 0);
        calender.set(Calendar.MILLISECOND, 0);
        calender.set(Calendar.MONTH, month);
        calender.set(Calendar.DAY_OF_MONTH, 1);
        _ResultDataProvider.setPeriod(calender, 1, Calendar.MONTH);
        export(lzz);
    }

    /**
     * verarbeitet intern Datens&auml;tze; darf nicht au&szlig;erhalb der Klasse
     * aufgerufen werden.
     *
     * @param rsArrayAtTime ResultData[]
     * @param calendar Calendar
     * @throws ChannelException
     */
    public void processRecord(Dataset[] rsArrayAtTime, Calendar calendar) throws ChannelException {
        try {

            // allg Check, ob Daten fr alls FS vorhanden
            if (rsArrayAtTime == null) {
                debug.finer("keine Daten");
            } else {
                int iFSWithOutData = 0;

                for (Dataset aRsArrayAtTime : rsArrayAtTime) {
                    if ((aRsArrayAtTime == null) || (aRsArrayAtTime.getData() == null)) {
                        iFSWithOutData++;
                    }
                }

                if (iFSWithOutData != 0) {
                    if (iFSWithOutData == rsArrayAtTime.length) {
                        debug.finer("alle Fahrstreifen ohne Daten");
                    } else {
                        debug.finer(String.valueOf(iFSWithOutData) + " Fahrstreifen ohne Daten");
                    }
                }
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }

        for (AbstractDataChannelBASt _Channel : _Channels) {
            boolean error = false;

            try {
                _Channel.processRecord(rsArrayAtTime, calendar);
            }
            catch (ChannelException e) {
                error = true;
                e.printStackTrace();

                throw new RuntimeException(e);
            }
            finally {
                if (error) {
                    System.exit(2);
                }
            }
        }
    }

    //~ SET METHODEN ==========================================================

    public void setAspect(Aspect asp) {
        DataModel      model = _dav.getDataModel();
        AttributeGroup atg   = model.getAttributeGroup("atg.verkehrsDatenLangZeitIntervall");

        _DataDescription = new DataDescription(atg, asp);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 06.08.12
