/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.kex.bast.exporter;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.inovat.kex.bast.SendeBAStStatusDaten;
import de.inovat.kex.bast.common.ChannelException;
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;

import java.io.*;
import java.util.ArrayList;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 *
 * <p>implementiert die Grundfunktionalit&auml;t f&uuml;r einen {@link de.inovat.kex.bast.common.ResultDataChannel ResultDataChannel},
 * der in eine Datei schreibt.</p>
 * <p>Durch {@link #reset(String, String)} reset} kann der Channel wieder verwendet
 * werden.</p>
 * <p>Erweiterung um die Mglichkeit der Ausgabe in eine Datei auf einem FTP-Server.
 * Dagmar Rostek (DR) inovat.</P>
 *
 *
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author R. Trier
 * @version $Id: FileDataChannel.java 452 2008-11-10 07:09:39Z Schawaller $
 */
abstract public class FileDataChannel implements de.inovat.kex.bast.common.ResultDataChannel {
    private ArrayList<Object> _LineBuffer = new ArrayList<Object>();
    private boolean           _IsOpen     = false;

    /** FTPClient fr die FTP Connetion. */
    private FTPClient _ftpClient = new FTPClient();

    /** Statusinformationen die an den Datenverteiler gesendet werden. */
    private SendeBAStStatusDaten _bastStatusDaten;

    // private static final de.inovat.kex.bast.funclib.logger.Logger logger =
    // de.inovat.kex.bast.funclib.logger.LoggerFactory.getLoggerFactory().getLogger(FileDataChannel.class);
    private String _FileName;

    /**
     * ======================================================================
     * Dateien zuknftig monatsweise in getrennten Ordnern abzulegen,
     * da die Datenmenge andernfalls schnell unbersichtlich werden drfte.
     * <p/>
     * BASt verwendet das Format [JJMM] fr die Benennung der Verzeichnisse
     * (zweistelliges Jahr, zweistelliger Monat 1-12).
     * ======================================================================
     *
     * Unterordner fr die FTP-Datei oder null.
     */
    private String _unterordnerName;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * erzeugt ein FileDataChannel. Zum Beschreiben des Channel muss mit
     * {@link #reset(String, String)} reset} der Filename gesetzt werden.
     *
     * @param bastStatusDaten Statusinformationen.
     */
    public FileDataChannel(SendeBAStStatusDaten bastStatusDaten) {
        _bastStatusDaten = bastStatusDaten;
    }

    //~ METHODEN ==============================================================

    /**
     * schlie&szlig;t den Channel nachdem gepufferte Daten geschrieben wurden.
     * Schliessen der FTPClient Verbindung nach der Datenbertragung. (Erweiterung
     * durch Dagmar Rostek (inovat)).
     * @throws ChannelException
     */
    public void close() throws ChannelException {
        if (_IsOpen) {
            flush();

            try {
                if (!(_ftpClient == null)) {
                    _ftpClient.disconnect();
                }
            }
            catch (IOException e) {
                e.printStackTrace();

                throw new RuntimeException(e);
            }
        }

        _LineBuffer.clear();
        _FileName        = null;
        _unterordnerName = null;
        _IsOpen          = false;
    }

    /**
     * l&ouml;scht die letzte Zeile aus dem Puffer.
     *
     * @throws ChannelException wenn der Channel geschlossen ist oder keine
     *   Zeilen im Puffer.
     */
    synchronized public void deleteLastLine() throws ChannelException {
        if ((_IsOpen) && (_LineBuffer.size() > 0)) {
            _LineBuffer.remove(_LineBuffer.size() - 1);
        } else {
            throw new ChannelException("Channel is closed");
        }
    }

    /**
     * Schreibt gepufferte Daten in die Datei.
     *
     * @throws ChannelException wenn beim Schreiben eine Exception erzeugt wird.
     * <p>
     *
     * Erweiterung der Methode um das Schreiben der gepufferten Daten auf den
     * FTP-Server. Wenn das Attribut _bastAusgabeAufFTPServer = true ist,
     * werden die gepufferten Daten auf den FTP-Server geschrieben. </p>
     * Erweiterung der Methode durch Dagmar Rostek (DR) inovat.
     */
    synchronized public void flush() throws ChannelException {
        FTPDataChannel ftpDataChannel = new FTPDataChannel(_FileName, _bastStatusDaten);
        boolean        loginOK;
        boolean        changeDirectoryOK;

        _ftpClient = ftpDataChannel.connectToFTPServerExport();

        if (_ftpClient != null) {
            loginOK = ftpDataChannel.loginToFTPServerExport();

            if (loginOK) {
                changeDirectoryOK = ftpDataChannel.changeExportDirektoryFTPServer(_unterordnerName);

                try {
                    if (FTPServerConnection.getInstance().isAchslastExport()) {
                        _ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
                    } else {
                        _ftpClient.setFileType(FTP.ASCII_FILE_TYPE);
                    }
                }
                catch (IOException e) {
                    throw new RuntimeException(e);
                }

                if (changeDirectoryOK) {
                    BufferedOutputStream bufferedAusgabe;
                    boolean              fileExistiert;

                    if (FTPServerConnection.getInstance().isDateiErsetzen()) {
                        ftpDataChannel.dateiLoeschen(_FileName);
                        bufferedAusgabe = ftpDataChannel.erzeugeAusgabeStrom();

                        try {
                            writeHeaderFTP(bufferedAusgabe);
                        }
                        catch (IOException e) {
                            e.printStackTrace();

                            throw new RuntimeException(e);
                        }

                        ftpDataChannel.schreibeDateiAufFTPServer(bufferedAusgabe, _LineBuffer);

                        try {
                            _bastStatusDaten.sendeBAStStatusDaten("", "Die Datei " + _FileName + " wurde erzeugt.");
                        }
                        catch (SendSubscriptionNotConfirmed ignored) {}
                    } else {
                        fileExistiert = ftpDataChannel.pruefeObFileExistiert(_FileName);

                        if (!fileExistiert) {
                            bufferedAusgabe = ftpDataChannel.erzeugeAusgabeStrom();

                            try {
                                writeHeaderFTP(bufferedAusgabe);
                            }
                            catch (IOException e) {
                                e.printStackTrace();

                                throw new RuntimeException(e);
                            }

                            ftpDataChannel.schreibeDateiAufFTPServer(bufferedAusgabe, _LineBuffer);
                        } else {
                            try {

                                // Statusmeldung an den Benutzer, dass die Datei nicht erzeugt wird, da sie schon existiert.
                                _bastStatusDaten.sendeBAStStatusDaten("", _FileName + ": Die Datei existiert schon, da das Flag vorhandene Date ersetzten nicht gesetzt ist, wird die Datei nicht exportiert.");
                            }
                            catch (SendSubscriptionNotConfirmed sendSubscriptionNotConfirmed) {
                                sendSubscriptionNotConfirmed.printStackTrace();

                                throw new RuntimeException(sendSubscriptionNotConfirmed);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * ist der Channel ge&ouml;ffnet, werden gepufferte Daten in die Datei
     * geschrieben und
     * danach die Datei mit dem angegebenen Namen zur Ausgabe benutzt.
     *
     * @param fileName String  .
     * @param ordnerName Unterordner fr die FTP-Datei (s. {@link #_unterordnerName}) oder null.
     *
     * @throws ChannelException    .
     */
    public void reset(String fileName, String ordnerName) throws ChannelException {
        if (_IsOpen) {
            close();
        }

        _FileName = fileName;
        _IsOpen   = true;

        //
        _unterordnerName = ordnerName;
    }

    /**
     * schreibt Daten in den Puffer.
     *
     * @param o zu schreibenes Object.
     * @throws ChannelException wenn der Channel geschlossen ist.
     */
    synchronized public void spool(Object o) throws ChannelException {
        if (_IsOpen) {
            _LineBuffer.add(o);
        } else {
            throw new ChannelException("Channel is closed");
        }
    }

    /**
     * erm&ouml;glicht das Schreiben von Daten am Anfang der Datei.
     *
     * @param writer PrintWriter
     * @throws IOException wenn beim Schreiben _Fehler auftreten.
     */
    abstract protected void writeHeader(PrintWriter writer) throws IOException;

    abstract protected void writeHeaderFTP(BufferedOutputStream bos) throws IOException;
}


//~Formatiert mit 'inovat Kodierkonvention' am 06.08.12
