/*
 * Copyright (c) 2010 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of importer.BastBestandsbandformatV2004
 *
 * de.inovat.kex.bast.importer.BastBestandsbandformatV2004 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * importer.BastBestandsbandformatV2004 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with .importer.BastBestandsbandformatV2004; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Kln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.kex.bast.importer;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.kex.bast.common.objectwrapper.Fahrstreifen;
import de.inovat.kex.bast.common.objectwrapper.LangZeitZaehlStelle;

import java.util.*;

//~ KLASSEN ===================================================================

/**
 * Die Klasse reprstentiert einen komplett eingelesenen Datensatz des
 * BASt-Bestandsbandformates Version 2004.
 * <p/>
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dagmar Rostek (DR)
 * @version $Revision: 318 $ / $Date: 2010-08-17 07:13:09 +0200 (Di, 17 Aug 2010) $ / ($Author: DR $)
 */
public class BastBestandsbandformat {
    private Zaehlstellenangaben                            _zaehlStellenAngaben                    = new Zaehlstellenangaben();
    private Richtungsangaben                               _richtungsAngaben                       = new Richtungsangaben();
    private Map<Fahrstreifen, ArrayList<Stundendatensatz>> _mapFSArrayListStundendatenSaetze8Plus1 = new HashMap<Fahrstreifen, ArrayList<Stundendatensatz>>();
    private Klassifikation                                 _klassifikation                         = new Klassifikation();

    //~ GET METHODEN ==========================================================

    public Klassifikation getKlassifikation() {
        return _klassifikation;
    }

    public Map<Fahrstreifen, ArrayList<Stundendatensatz>> getMapFSArrayListStundendatenSaetze8Plus1() {
        return _mapFSArrayListStundendatenSaetze8Plus1;
    }

    public Richtungsangaben getRichtungsAngaben() {
        return _richtungsAngaben;
    }

    /**
     * Getter des Attributes Zaehlstellenangaben, welches einer Instanz der Klasse
     * Zaehlstellenangaben entspricht.
     * @return Instanz der Klasse Zaehlstellenangeben.
     */
    public Zaehlstellenangaben getZaehlStellenAngaben() {
        return _zaehlStellenAngaben;
    }
}


/**
 * Die Klasse reprstentiert alle Informationen zu den Richtungsangaben, welches dem zweiten
 * Header-Datensatz der Importdatei entspricht.
 */
class Richtungsangaben {
    private String _anzahlFSRichtungI;
    private String _anzahlFSRichtungII;
    private String _fernzielRichtungI;
    private String _fernzielRichtungII;
    private String _headerkennung;
    private String _himmelsrichtungI;
    private String _himmelsrichtungII;
    private String _nahzielRichtungI;
    private String _nahzielRichtungII;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    Richtungsangaben() {}

    //~ SET METHODEN ==========================================================

    public void setAnzahlFSRichtungI(String anzahlFSRichtungI) {
        _anzahlFSRichtungI = anzahlFSRichtungI;
    }

    public void setAnzahlFSRichtungII(String anzahlFSRichtungII) {
        _anzahlFSRichtungII = anzahlFSRichtungII;
    }

    public void setFernzielRichtungI(String fernzielRichtungI) {
        _fernzielRichtungI = fernzielRichtungI;
    }

    public void setFernzielRichtungII(String fernzielRichtungII) {
        _fernzielRichtungII = fernzielRichtungII;
    }

    public void setHeaderkennung(String headerkennung) {
        _headerkennung = headerkennung;
    }

    public void setHimmelsrichtungI(String himmelsrichtungI) {
        _himmelsrichtungI = himmelsrichtungI;
    }

    public void setHimmelsrichtungII(String himmelsrichtungII) {
        _himmelsrichtungII = himmelsrichtungII;
    }

    public void setNahzielRichtungI(String _nahzielRichtungI) {
        this._nahzielRichtungI = _nahzielRichtungI;
    }

    public void setNahzielRichtungII(String _nahzielRichtungII) {
        this._nahzielRichtungII = _nahzielRichtungII;
    }

    //~ METHODEN ==============================================================

    @Override
    public String toString() {
        return "Richtungsangaben{" + "_headerkennung='" + _headerkennung + '\'' + ", _anzahlFSRichtungI='" + _anzahlFSRichtungI + '\'' + ", _anzahlFSRichtungII='" + _anzahlFSRichtungII + '\'' + ", _fernzielRichtungI='" + _fernzielRichtungI + '\'' + ", _himmelsrichtungI='" + _himmelsrichtungI + '\'' + ", _fernzielRichtungII='" + _fernzielRichtungII + '\'' + ", _himmelsrichtungII='" + _himmelsrichtungII + '\'' + '}';
    }
}




/**
 * Die Klasse reprstentiert alle Informationen zu den Zhlstellenangaben, welches dem ersten
 * Header-Datensatz der Importdatei entspricht.
 */
class Zaehlstellenangaben {
    private String              _blattNr;
    private String              _bundesLandKennung;
    private String              _headerkennung;
    private LangZeitZaehlStelle _lzz;
    private String              _lzzName;
    private String              _lzzNr;
    private String              _strassenKlasse;
    private String              _strassenNr;
    private String              _versionsNummer;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    Zaehlstellenangaben() {}

    //~ GET METHODEN ==========================================================



    public LangZeitZaehlStelle getLzz() {
        return _lzz;
    }



    //~ SET METHODEN ==========================================================

    public void setBlattNr(String blattNr) {
        _blattNr = blattNr;
    }

    public void setBundesLandKennung(String bundesLandKennung) {
        _bundesLandKennung = bundesLandKennung;
    }

    public void setHeaderkennung(String headerkennung) {
        _headerkennung = headerkennung;
    }

    public void setLzz(LangZeitZaehlStelle lzz) {
        _lzz = lzz;
    }

    public void setLzzName(String lzzName) {
        _lzzName = lzzName;
    }

    public void setLzzNr(String lzzNr) {
        _lzzNr = lzzNr;
    }

    public void setStrassenKlasse(String strassenKlasse) {
        _strassenKlasse = strassenKlasse;
    }

    public void setStrassenNr(String strassenNr) {
        _strassenNr = strassenNr;
    }

    public void setVersionsNummer(String versionsNummer) {
        _versionsNummer = versionsNummer;
    }

    //~ METHODEN ==============================================================

    @Override
    public String toString() {
        return "Zaehlstellenangaben{" + "_headerkennung='" + _headerkennung + '\'' + ", _blattNr='" + _blattNr + '\'' + ", _lzzNr='" + _lzzNr + '\'' + ", _bundesLandKennung='" + _bundesLandKennung + '\'' + ", _strassenKlasse='" + _strassenKlasse + '\'' + ", _strassenNr='" + _strassenNr + '\'' + ", _lzzName='" + _lzzName + '\'' + ", _versionsNummer='" + _versionsNummer + '\'' + '}';
    }

    //~ INNERE KLASSEN ========================================================


}


//~Formatiert mit 'inovat Kodierkonvention' am 27.08.10
